@@ -192,7 +192,6 @@ class AssetSerializer(NeedsReviewMixin, HistorySerializerMixin):
 
     def create(self, validated_data):
         data = self._get_instance(validated_data)
-        data['last_updated'] = timezone.now()
         asset = super(AssetSerializer, self).create(data)
         if asset['asset_type'] == 'vm' and asset['state'] == 'ready':
             from socrates_api.tasks import provision_vm, add_to_dns, ipmi_poweron
@@ -210,7 +209,6 @@ class AssetSerializer(NeedsReviewMixin, HistorySerializerMixin):
         if not diff and 'latest_action' not in validated_data:
             return instance
 
-        update['last_updated'] = timezone.now()
         asset = super(AssetSerializer, self).update(instance, validated_data)
         old_asset = instance
 
@@ -306,7 +304,6 @@ class AssetSerializer(NeedsReviewMixin, HistorySerializerMixin):
                'version': self.instance['version'],
                'decommissioning': True,
                'provisioning': False,
-               'last_updated': timezone.now()
            })
         task.apply_async((asset,))
         return True
@@ -334,16 +331,6 @@ class AssetSerializer(NeedsReviewMixin, HistorySerializerMixin):
                 raise serializers.ValidationError('version=%d is not the expected %d' % (value, self.instance['version']))
         return value
 
-    def validate_created(self, value):
-        if isinstance(value, str):
-            value = datetime.datetime.strptime(value[:19], "%Y-%m-%dT%H:%M:%S").replace(tzinfo=timezone.utc)
-        if self.instance is not None and 'created' in self.instance:
-            if isinstance(self.instance['created'], str):
-                self.instance['created'] = datetime.datetime.strptime(self.instance['created'][:19], "%Y-%m-%dT%H:%M:%S").replace(tzinfo=timezone.utc)
-            if value.strftime("%Y-%m-%dT%H:%M:%S") != self.instance['created'].strftime("%Y-%m-%dT%H:%M:%S"):
-                raise serializers.ValidationError("time travelling denied")
-        return value
-
     def validate(self, data):
         from socrates_api.tasks import get_ipam, ipv4_network_contains
         if 'log' not in data:
@@ -557,6 +544,7 @@ class OperatingSystemSerializer(RethinkSerializer):
     kickstart = serializers.CharField(required=False, trim_whitespace=False)
     ids = serializers.DictField(required=False)
     minimums = serializers.DictField(required=False)
+    tags = serializers.DictField(required=False)
 
     class Meta(RethinkSerializer.Meta):
         table_name = 'os'
@@ -592,20 +580,12 @@ class FirewallAddressSerializer(serializers.Serializer):
             raise serializers.ValidationError("only one of address, address_group, fqdn, and user_groups can be set")
         return data
 
-    def resolve(self):
-        if 'address' in self.instance:
-            return [self.instance.copy()]
-        elif 'address_group' in self.instance:
-            ret = []
-            for address in FirewallAddressGroupSerializer.get(name=self.instance['address_group'])['addresses']:
-                ret.extend(FirewallAddressSerializer(address).resolve())
-            return ret
-
 class FirewallAddressGroupSerializer(HistorySerializerMixin):
     id = serializers.CharField(required=False)
     name = serializers.CharField(required=True)
     addresses = serializers.ListField(child=FirewallAddressSerializer(), required=True)
     permissions = PermissionsSerializer(required=False)
+    tags = serializers.DictField(required=False)
 
     class Meta(RethinkSerializer.Meta):
         table_name = 'firewall_address_group'
@@ -661,6 +641,7 @@ class FirewallRuleSetSerializer(HistorySerializerMixin):
     permissions = PermissionsSerializer(required=False)
     rulesets = serializers.ListField(child=serializers.CharField(validators=[validate_ruleset_name]), required=False)
     rules = serializers.ListField(child=FirewallRuleSerializer(), required=True)
+    tags = serializers.DictField(required=False)
 
     class Meta(RethinkSerializer.Meta):
         table_name = 'firewall_rule_set'
@@ -671,14 +652,6 @@ class FirewallRuleSetSerializer(HistorySerializerMixin):
             ('permissions_write', r.row['permissions']['write'], {'multi': True}),
         ]
 
-    def resolve(self):
-        rules = []
-        for ruleset_name in self.instance.get('rulesets', []):
-            ruleset = FirewallRuleSetSerializer(FirewallRuleSetSerializer.get(name=ruleset_name))
-            rules.extend(ruleset.resolve())
-        rules.extend(self.instance['rules'])
-        return rules
-
     def update(self, instance, data):
         from socrates_api.tasks import firewall_apply
         ruleset = super(FirewallRuleSetSerializer, self).update(instance, data)
@@ -701,7 +674,7 @@ class FirewallRuleSetSerializer(HistorySerializerMixin):
                     pass
             else:
                 continue
-            firewall_apply.apply_async((asset, network))
+            firewall_apply.apply_async((asset,))
         return ruleset
 
     def create_link(self, instance):
@@ -722,6 +695,7 @@ class NetworkSerializer(NeedsReviewMixin, HistorySerializerMixin):
     needs_review = serializers.BooleanField(required=False)
     domains = serializers.DictField(child=NetworkDomainSerializer(), required=True)
     ruleset = serializers.CharField(validators=[validate_ruleset_name], required=False)
+    tags = serializers.DictField(required=False)
 
     class Meta(RethinkSerializer.Meta):
         table_name = 'networks'
@@ -766,7 +740,7 @@ class NetworkSerializer(NeedsReviewMixin, HistorySerializerMixin):
                 except:
                     pass
                 else:
-                    firewall_apply.apply_async((asset, network))
+                    firewall_apply.apply_async((asset,))
         return network
 
     def delete(self):
@@ -899,6 +873,7 @@ class QuotaSerializer(RethinkSerializer):
     total_ram = serializers.IntegerField(required=True)
     vm_disk = serializers.IntegerField(required=True)
     total_disk = serializers.IntegerField(required=True)
+    tags = serializers.DictField(required=False)
 
     class Meta(RethinkSerializer.Meta):
         table_name = 'quotas'
@@ -973,6 +948,7 @@ class LoadBalancerIRuleSerializer(HistorySerializerMixin):
     code = serializers.CharField(required=True)
     pools = serializers.ListField(child=LoadBalancerIRulePoolSerializer(), required=False)
     permissions = PermissionsSerializer(required=False)
+    tags = serializers.DictField(required=False)
 
     class Meta(RethinkSerializer.Meta):
         table_name = 'load_balancer_irule'
@@ -1050,6 +1026,7 @@ class LoadBalancerSerializer(HistorySerializerMixin):
     lb_method = serializers.ChoiceField(choices=LOAD_BALANCING_METHODS, required=False)
     permissions = PermissionsSerializer(required=False)
     groups = serializers.ListField(child=LoadBalancerGroupSerializer(), required=False)
+    tags = serializers.DictField(required=False)
 
     class Meta(RethinkSerializer.Meta):
         table_name = 'load_balancer'
